from src.logic import GenomeDownloader
from src.logic import Splitter
from src.logic import Blast
from src.logic import GenomeMapping
from src.logic import PrimersMaking
from src.logic import GenomeMapping



import wx


class MainPanel(wx.Panel):
    def __init__(self, parent):
        wx.Panel.__init__(self, parent=parent)
        vbox = wx.BoxSizer(wx.VERTICAL)

        vbox.Add(wx.Size(100, 50))
        prepare = wx.BoxSizer(wx.HORIZONTAL, )
        blast = wx.BoxSizer(wx.HORIZONTAL)
        primer = wx.BoxSizer(wx.HORIZONTAL)
        info = wx.BoxSizer(wx.HORIZONTAL)

        vbox.Add(prepare, flag=wx.ALIGN_CENTER)
        vbox.Add(wx.Size(20, 50))
        vbox.Add(blast, flag=wx.ALIGN_CENTER)
        vbox.Add(wx.Size(20, 50))
        vbox.Add(primer, flag=wx.ALIGN_CENTER)
        vbox.Add(wx.Size(40, 50))
        vbox.Add(info, flag=wx.LEFT | wx.BOTTOM)

        self.btn_prepare = wx.Button(self, wx.ID_ANY, label="Step 1: Data preparation")
        self.btn_blast = wx.Button(self, wx.ID_ANY, label="Step 2: Search for species-specific areas")
        self.btn_primer = wx.Button(self, wx.ID_ANY,
                                    label="Step 3: Selection of fragments for the construction of PCR primers")
        btn_info = wx.Button(self, wx.ID_ANY, label='Info')
        btn_info.Bind(wx.EVT_BUTTON, self.get_info)

        prepare.Add(self.btn_prepare)
        blast.Add(self.btn_blast)
        primer.Add(self.btn_primer)
        info.Add(btn_info, flag=wx.ALL, border=8)
        self.SetSizer(vbox)
        self.Layout()

    def get_info(self, event):
        info_window = InfoWindow(parent=wx.Panel(self))
        info_window.Show()


class InfoWindow(wx.Frame):
    def __init__(self, parent):
        super().__init__(parent, title='Info', size=(600, 450))
        panel = wx.Panel(self)
        text = '\n' + '   At the first step, all genomes of the target genus are downloaded from the NCBI database' + '\n' +\
               '   and two local databases, so called “positive” and “negative” databases, are formed. ' + '\n' + '\n' +\
               '   At the second step, the search for unique sequences is carried out and mapping finding ' + '\n' +\
               '   sequences on target genome. ' + '\n' + '\n' +\
               '   At the last step the program makes primers'
        static_text = wx.StaticText(panel, wx.ID_ANY, label=text, style=wx.ALIGN_LEFT)
        font = wx.Font(14, wx.FONTFAMILY_SWISS, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_NORMAL)
        static_text.SetFont(font)



from Bio.Blast.Applications import NcbiblastnCommandline
from Bio import SeqIO
from Bio.Blast import NCBIXML
import os


class Blast:
    def __init__(self, input, db, output, format, evalue, num_threads, task):
        sequence_file = open(input, 'r')
        sequence_lines = sequence_file.readlines()
        records = SeqIO.to_dict(SeqIO.parse(input, format))

        # Count of sequences in file for blast
        sequences_per_chunk = 1000
        line_per_sequence = round(len(sequence_lines) / len(records))

        if not os.path.exists(output):
            os.mkdir(output)

        # Results files 'hits' and 'no_hits'
        hits_file = open(output + '/' + 'hits.fna', 'w')
        no_hits_file = open(output + '/' + 'no_hits.fna', 'w')

        # Function that spares a array into subareas
        def split_list(array, length):
            return [array[i:i + length] for i in range(0, len(array), length)]

        # Array of string (1000 sequences + 1000 names)
        chunks = split_list(sequence_lines, sequences_per_chunk * line_per_sequence)
        chunk_name = 'chunk.fna'
        chunk_result_file_name = 'chunk_result.xml'

        # For statistics
        records_count = 0
        no_hits_count = 0
        hits_count = 0

        # loop through subareas of 1000 sequences
        # a temporary file is created to which the current 1000 sequences are written
        for i, chunk in enumerate(chunks):
            chunk_file = open(chunk_name, 'w')
            text = ''.join(chunk)
            chunk_file.write(text)

            # creates a temporary file for output, which is overwritten each time

            # выбор BLAST и e-value
            blastn_cline = NcbiblastnCommandline(task=task, query=chunk_name, db=db, evalue=evalue,
                                                 outfmt=5, out=chunk_result_file_name, num_threads=num_threads)
            stdout, stderr = blastn_cline()

            # algorithm for finding hits and non-hits from each 1000 sequences
            q_dict = SeqIO.index(chunk_name, 'fasta')

            hits = []

            for record in NCBIXML.parse(open(chunk_result_file_name)):

                if record.alignments:
                    hits.append(record.query.split()[0])

            no_hits = set(q_dict.keys()) - set(hits)
            orphan_records = [q_dict[name] for name in no_hits]

            # for checking the algorithm
            records_count += len(q_dict)
            hits_count += len(hits)
            no_hits_count += len(no_hits)

            for hit in hits:
                hits_file.write(q_dict[hit].format('fasta'))

            for no_hit in no_hits:
                no_hits_file.write(q_dict[no_hit].format('fasta'))

            chunk_file.close()

            print(i + 1, ' from ', len(chunks), ':',
                  "found %i records in query, %i have hits, making %i no hits" % (
                  records_count, hits_count, no_hits_count))

        os.remove(chunk_name)
        os.remove(chunk_result_file_name)

        print('Complete!')



#!/usr/bin/env python
import argparse
from .logic import Blast


def run_blast():
    parser = argparse.ArgumentParser()
    parser.add_argument('--input', dest='input', type=str, help='Path to input file', required=True)
    parser.add_argument('--db', dest='db', type=str, help='Path to blast database', required=True)
    parser.add_argument('--output', dest='output', type=str, default='blast_result', help='Path to output folder')
    parser.add_argument('--format', dest='format', default='fasta', type=str, help='Format of input file')
    parser.add_argument('--evalue', dest='evalue', default=10, type=int,
                        help='The BLAST E-value is the number of expected hits of similar quality (score) that could be found just by chance. ')
    parser.add_argument('--num_threads', dest='num_threads', type=int,
                        help='Number of threads (CPUs) to use in blast search')
    parser.add_argument('--task', dest='task', default='blastn', type=str,
                        help='Four different tasks are supported: 1) “megablast”, for very similar sequences (e.g, sequencing errors), 2) “dc-megablast”, typically used for inter-species comparisons, 3) “blastn”, the traditional program used for inter-species comparisons, 4) “blastn-short”, optimized for sequences less than 30 nucleotides.')

    args = parser.parse_args()
    return args.input, args.db, args.output, args.format, args.evalue, args.num_threads, args.task


input, db, output, format, e_value, num_threads, task = run_blast()

Blast(input, db, output, format, e_value, num_threads, task)



from Bio import SeqIO
import re
import os


# function, that find start of interval
def get_interval_start(record_id):
    interval = re.search(r'\d*-\d*', record_id).group(0)
    interval_start = re.search(r'\d*', interval).group(0)
    return int(interval_start)


# function, that find end of interval
def get_interval_end(record_id):
    interval = re.search(r'\d*-\d*', record_id).group(0)
    interval_end = re.findall(r'\d*\w', interval)[1]
    return int(interval_end)


def concat_records(first_record, second_record):
    step = get_interval_start(second_record.id) - get_interval_start(first_record.id)
    result_seq = first_record.seq[:step] + second_record.seq
    result_id = first_record.id.replace(str(get_interval_end(first_record.id)),
                                        str(get_interval_end(second_record.id)))
    result_description = first_record.description.replace(first_record.id, result_id)
    return SeqIO.SeqRecord(result_seq, result_id, result_id, result_description)


class GenomeMapping:
    def __init__(self, input_file, output_folder):
        # make directory for result
        if not os.path.exists(output_folder):
            os.mkdir(output_folder)

        dict_of_records = {}

        with open(input_file) as handle:
            for i, record in enumerate(SeqIO.parse(handle, "fasta")):
                dict_of_records[get_interval_start(record.id)] = record

            list_key = list(dict_of_records.keys())
            list_key.sort()

            current_record = None
            is_end_of_match = False

            for i, key in enumerate(list_key):
                if i < len(list_key) - 1 and list_key[i + 1] < \
                        get_interval_end(dict_of_records[list_key[i]].id):
                    if not current_record:
                        current_record = dict_of_records[key]
                    else:
                        current_record = concat_records(current_record, dict_of_records[key])
                    is_end_of_match = True
                else:
                    if is_end_of_match:
                        current_record = concat_records(current_record, dict_of_records[key])
                        file_name = output_folder + '/' + 'file_' + str(get_interval_start(current_record.id)) + \
                                    '_' + str(get_interval_end(current_record.id))
                        file = open(file_name, 'w')
                        file.write('>' + current_record.description + '\n' + str(current_record.seq) + '\n')
                        is_end_of_match = False
                        current_record = None



import os
import primer3
from Bio import SeqIO


class PrimersMaking:
    def __init__(self, input_folder, output_folder, length, opt, min, max, opt_tm, min_tm, max_tm):
        if not os.path.exists(output_folder):
            os.mkdir(output_folder)

        list_of_files = os.listdir(input_folder)

        sequence_index = 1

        primer = {}

        for file_name in list_of_files:
            path_to_file = input_folder + '/' + file_name
            file = open(path_to_file, 'r')
            with file as handle:
                for record in SeqIO.parse(handle, 'fasta'):
                    seq_id = record.id
                    seq = str(record.seq)

                if len(seq) >= length:
                    primer = primer3.bindings.designPrimers(
                        {
                            'SEQUENCE_ID': seq_id,
                            'SEQUENCE_TEMPLATE': seq,
                        },
                        {
                            'PRIMER_OPT_SIZE': opt,
                            'PRIMER_MIN_SIZE': min,
                            'PRIMER_MAX_SIZE': max,
                            'PRIMER_OPT_TM': opt_tm,
                            'PRIMER_MIN_TM': min_tm,
                            'PRIMER_MAX_TM': max_tm,
                        })

                    with open(output_folder + '/' + 'primer_' + str(sequence_index), 'w') as out:
                        out.write(record.description + '\n' + '\n')
                        for key, val in primer.items():

                            out.write('{}:{}\n'.format(key, val))
                        sequence_index += 1



import wx
from src.logic import Blast


class BlastPanel(wx.Panel):
    def __init__(self, parent):
        wx.Panel.__init__(self, parent=parent)
        vbox = wx.BoxSizer(wx.VERTICAL)
        self.SetSizer(vbox)

        hbox1 = wx.BoxSizer(wx.HORIZONTAL)
        hbox2 = wx.BoxSizer(wx.HORIZONTAL)
        hbox3 = wx.BoxSizer(wx.HORIZONTAL)
        hbox4 = wx.BoxSizer(wx.HORIZONTAL)
        hbox5 = wx.BoxSizer(wx.HORIZONTAL)
        hbox6 = wx.BoxSizer(wx.HORIZONTAL)

        st1 = wx.StaticText(self, label='input')
        hbox1.Add(st1, flag=wx.RIGHT, border=8)

        btn_file = wx.Button(self, label='choose file', size=(90, 30))
        hbox1.Add(btn_file, flag=wx.RIGHT, border=8)
        btn_file.Bind(wx.EVT_BUTTON, self.choose_file)

        self.input = wx.TextCtrl(self)
        hbox1.Add(self.input, proportion=1)
        vbox.Add(hbox1, flag=wx.EXPAND | wx.LEFT | wx.RIGHT | wx.TOP, border=10)

        st2 = wx.StaticText(self, label='database')
        hbox2.Add(st2, flag=wx.RIGHT, border=8)
        btn_file = wx.Button(self, label='choose folder', size=(100, 30))
        hbox2.Add(btn_file, flag=wx.RIGHT, border=8)
        btn_file.Bind(wx.EVT_BUTTON, self.choose_folder_database)

        self.database = wx.TextCtrl(self)
        hbox2.Add(self.database, proportion=1)
        vbox.Add(hbox2, flag=wx.EXPAND | wx.LEFT | wx.RIGHT | wx.TOP, border=10)

        st3 = wx.StaticText(self, label='output')
        hbox3.Add(st3, flag=wx.RIGHT, border=8)

        btn_file = wx.Button(self, label='choose folder', size=(100, 30))
        hbox3.Add(btn_file, flag=wx.RIGHT, border=8)
        btn_file.Bind(wx.EVT_BUTTON, self.choose_folder_output)

        self.output = wx.TextCtrl(self)
        hbox3.Add(self.output, proportion=1)
        vbox.Add(hbox3, flag=wx.EXPAND | wx.LEFT | wx.RIGHT | wx.TOP, border=10)

        st4 = wx.StaticText(self, label='e-value')
        self.evalue = wx.TextCtrl(self, value='10')
        hbox4.Add(st4, flag=wx.RIGHT, border=8)
        hbox4.Add(self.evalue, proportion=1)
        vbox.Add(hbox4, flag=wx.EXPAND | wx.LEFT | wx.RIGHT | wx.TOP, border=10)

        st5 = wx.StaticText(self, label='num_threads')
        self.num_threads = wx.TextCtrl(self)
        hbox5.Add(st5, flag=wx.RIGHT, border=8)
        hbox5.Add(self.num_threads, proportion=1)
        vbox.Add(hbox5, flag=wx.EXPAND | wx.LEFT | wx.RIGHT | wx.TOP, border=10)

        tasks_list = ['blastn', 'dc-megablast', 'megablast', 'blastn-short']

        self.rbox = wx.RadioBox(self, label='Task', choices=tasks_list,
                                majorDimension=1, style=wx.RA_SPECIFY_ROWS)
        hbox6.Add(self.rbox)
        vbox.Add(hbox6)

        vbox.Add(wx.Size(50, 50))

        self.btnRun = wx.Button(self, label='RUN', size=(70, 30))
        self.btnReturn = wx.Button(self, label='Return to prepare')
        h_btn = wx.BoxSizer(wx.HORIZONTAL)
        h_btn.Add(self.btnReturn, flag=wx.LEFT, border=10)
        h_btn.Add(wx.Size(450, 10))
        h_btn.Add(self.btnRun, flag=wx.LEFT, border=10)
        vbox.Add(h_btn, flag=wx.ALIGN_LEFT | wx.BOTTOM | wx.LEFT, border=10)

    def choose_file(self, event):
        dlg = wx.FileDialog(
            self, message="Choose a file")
        if dlg.ShowModal() == wx.ID_OK:
            paths = dlg.GetPath()
            self.input.SetValue(str(paths))
        dlg.Destroy()

    def choose_folder_output(self, event):
        dlg = wx.DirDialog(
            self, message="Choose a folder")
        if dlg.ShowModal() == wx.ID_OK:
            paths = dlg.GetPath()
            self.output.SetValue(str(paths))
        dlg.Destroy()

    def choose_folder_database(self, event):
        dlg = wx.DirDialog(
            self, message="Choose a folder")
        if dlg.ShowModal() == wx.ID_OK:
            paths = dlg.GetPath()
            self.database.SetValue(str(paths))
        dlg.Destroy()



#!/usr/bin/env python
import argparse
from .logic import PrimersMaking


def make_primers():
    parser = argparse.ArgumentParser()
    parser.add_argument('--input', dest='input', type=str, help='Path to input file', required=True)
    parser.add_argument('--output', dest='output', type=str, default='file_with_primers', help='Path to output folder')
    parser.add_argument('--length', dest='length', type=int, help='The length of the sequence from which the primers will be selected')
    parser.add_argument('--opt_primer', dest='opt', type=int, help='Optimum length (in bases) of a primer. Primer3 will attempt to pick primers close to this length')
    parser.add_argument('--min_primer', dest='min', type=int, help='Minimum acceptable length of a primer. Must be greater than 0 and less than or equal to max_primer')
    parser.add_argument('--max_primer', dest='max', type=int, help='Maximum acceptable length (in bases) of a primer. Currently this parameter cannot be larger than 35')
    parser.add_argument('--opt_tm', dest='opt_tm', default=60.0, type=float, help='Optimum melting temperature (Celsius) for a primer. Default = 60.0')
    parser.add_argument('--min_tm', dest='min_tm', default=57.0, type=float, help='Minimum acceptable melting temperature (Celsius) for a primer oligo. Default = 57.0')
    parser.add_argument('--max_tm', dest='max_tm', default=63.0, type=float, help='Maximum acceptable melting temperature (Celsius) for a primer oligo. Default = 63.0')
    args = parser.parse_args()
    return args.input, args.output, args.length, args.opt, args.min, args.max, args.opt_tm, args.min_tm, args.max_tm


input, output, length, opt, min, max, opt_tm, min_tm, max_tm = make_primers()
PrimersMaking(input, output, length, opt, min, max, opt_tm, min_tm, max_tm)



#!/usr/bin/env python
import argparse
from .logic import GenomeDownloader


def make_prepare():
    parser = argparse.ArgumentParser()
    parser.add_argument('-g', '--genus', dest='genus', type=str, help='Bacteria genus. For example: pectobacterium',
                        required=True)
    parser.add_argument('-s', '--species', dest='species', type=str, help='Bacteria species. For example: parmentieri',
                        required=True)
    parser.add_argument('-i', '--id', dest='id', type=str, help='INSDC from NCBI', required=True)
    parser.add_argument('-o', '--output', dest='output', type=str, default='download', help='path to output folder')
    parser.add_argument('-ps', '--positive_strain', dest='positive_strain', type=str,
                        help='Strains that are misclassified in the negative database. Must be classified as a positive database.')
    parser.add_argument('-ns', '--negative_strain', dest='negative_strain', type=str,
                        help='Strains that are misclassified in the positive database. Must be classified as a negative database.')

    args = parser.parse_args()
    return args.output, args.genus, args.species, args.id


output, genus, species, id = make_prepare()

GenomeDownloader(output, genus, species, id)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['src', 'src.gui', 'src.logic']

package_data = \
{'': ['*']}

install_requires = \
['biopython>=1.79,<2.0',
 'logic>=0.2.3,<0.3.0',
 'ncbi-genome-download>=0.3.1,<0.4.0',
 'primer3-py>=0.6.1,<0.7.0',
 'requests>=2.26.0,<3.0.0',
 'wxPython>=4.1.1,<5.0.0']

entry_points = \
{'console_scripts': ['blast = src.blast:run_blast',
                     'mapping = src.mapping:mapping_genome',
                     'prepare = src.prepare:make_prepare',
                     'primer = src.primer_3:make_primers',
                     'splitter = src.splitter:make_split_sequence',
                     'start-gui = src.run_gui:run_my_app']}

setup_kwargs = {
    'name': 's3finder',
    'version': '0.1.4',
    'description': 'Light and fast Python pipeline for searching species-specific regions in bacterial genomes',
    'long_description': None,
    'author': 'Polina Rasskazova',
    'author_email': 'rasskazova.pm@phystech.edu',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import wx
from .splitter_window import SplitterWindow
from .prepare_window import PrepareWindow


class PreparePanel(wx.Panel):
    def __init__(self, parent):
        wx.Panel.__init__(self, parent=parent)
        vbox = wx.BoxSizer(wx.VERTICAL)

        vbox.Add(wx.Size(50, 50))
        hbox1 = wx.BoxSizer(wx.HORIZONTAL)
        hbox2 = wx.BoxSizer(wx.HORIZONTAL)
        statictext1 = wx.StaticText(self, wx.ID_ANY, label="Do you need to download genomes and create a database?")
        statictext2 = wx.StaticText(self, wx.ID_ANY, label="Do you need to split the genome?")

        hbox1.Add(statictext1, flag=wx.ALIGN_CENTER)
        hbox2.Add(statictext2, flag=wx.ALIGN_CENTER)

        vbox.Add(hbox1, flag=wx.ALIGN_CENTER)
        vbox.Add(hbox2, flag=wx.ALIGN_CENTER)

        btn_yes1 = wx.Button(self, wx.ID_ANY, label="Yes", size=(50, 30))
        btn_yes2 = wx.Button(self, wx.ID_ANY, label="Yes", size=(50, 30))

        hbox1.Add(btn_yes1, flag=wx.ALL, border=10)
        hbox2.Add(btn_yes2, flag=wx.ALL, border=10)

        vbox.Add(wx.Size(100, 100))

        btn_yes1.Bind(wx.EVT_BUTTON, self.OpenPrepareWindow)
        btn_yes2.Bind(wx.EVT_BUTTON, self.OpenSplitterWindow)

        # кнопка RUN
        self.btnRun = wx.Button(self, label='To Blast Step')
        hbox3 = wx.BoxSizer(wx.HORIZONTAL)
        hbox3.Add(self.btnRun, flag=wx.LEFT, border=10)
        vbox.Add(hbox3, flag=wx.ALIGN_RIGHT | wx.BOTTOM | wx.RIGHT, border=10)
        self.SetSizer(vbox)

    def OpenPrepareWindow(self, event):
        secondWindow = PrepareWindow(parent=wx.Panel(self))
        secondWindow.Show()

    def OpenSplitterWindow(self, event):
        thirdWindow = SplitterWindow(parent=wx.Panel(self))
        thirdWindow.Show()



#!/usr/bin/env python
import argparse
from .logic import Splitter


def make_split_sequence():
    parser = argparse.ArgumentParser()
    parser.add_argument('-s', '--sequence', dest='sequence', type=str, help='Path to file with sequence', required=True)
    parser.add_argument('-l', '--length', dest='length', type=int,
                        help='The length of the fragments into which input sequence will be cut', required=True)
    parser.add_argument('-i', '--interval', dest='interval', type=int,
                        help='The interval at which the fragments will be cut', required=True)
    parser.add_argument('-o', '--output', dest='output', type=str, default='split_sequence', help='Path to output file')

    args = parser.parse_args()
    return args.output, args.sequence, args.length, args.interval


output, sequence, length, interval = make_split_sequence()

Splitter(output, sequence, length, interval)
